/****************************************************************************
   File:        Lock.java
   Description: Software to test the scheduler.
   Project:     MAC 322 - Distributed Scheduler
   Authors:     Nelio Pereira (nelio@ime.usp.br)
   Institution: Department of Computer Science
                University of Sao Paulo
******************************************************************************/

package sod.exec;

import java.io.Serializable;

/**
 * MUTEX.
 * Object that is used to lock processes.
 * Created in July 30, 2001.<BR>
 * 
 * <BR>
 * @author <a href="mailto:nelio@ime.usp.br">Nlio Alves Pereira Filho</a><BR>
 * @author <a href="mailto:riko@ime.usp.br">Ricardo Koji Ushizaki</a>
 * @version 1.0 June/2001
 */

public class Lock implements Serializable {

    private boolean threadSleep;

    /**
     * Constructs a new lock.
     */
    public Lock() {
	threadSleep = false;
    }
    
    /**
     * Acquires the lock. Keep waiting until lock is achieved.
     * @throws InterruptedException Some error occurs while waiting for lock.
     */
    public synchronized void lock() throws InterruptedException {
	if (threadSleep) {
	    wait();
	}
	threadSleep = true;
    }
    
    /**
     * Releases the lock, and notify only one thread.
     */
    public synchronized void unlock() {
	threadSleep = false;
	notify();
    }

}
