/****************************************************************************
   File:        LoadAdapter.java
   Description: Responsible for adapting the load at the processors, migrating processes if necessary.
   Project:     MAC 322 - Distributed Scheduler
   Authors:     Nelio Pereira (nelio@ime.usp.br)
   Institution: Department of Computer Science
                University of Sao Paulo
******************************************************************************/

package sod.server;

import sod.util.Process;
import sod.util.Processor;
import java.rmi.RemoteException;

/**
 *
 * Responsible for adapting the load at the processor, migrating processes if necessary.
 */
class LoadAdapter extends Thread {
    
    private Monitor monitor;
    private Scheduler scheduler;
    private int mseconds;
    
    public LoadAdapter(Monitor monitor, Scheduler scheduler, int seconds) {
	this.monitor = monitor;
	this.scheduler = scheduler;
	this.mseconds = seconds*1000;
    }
    
    public void run() {
	while (true) {
	    try {
		String[] hosts = scheduler.getHosts();
		if (hosts.length > 0) {
		    float averageLoad = (float)scheduler.getNumProcesses()/hosts.length;
		    for (int i=0; i < hosts.length; i++) {
			Processor processor = scheduler.getProcessor(hosts[i]);
			int score = processor.getScore();
			int numProcesses = (int)(score-averageLoad);
			if (numProcesses > 0) {
			    int[] pids = processor.getPIDs(numProcesses);
			    for (int j=0; j < numProcesses; j++) {
				Process newProcess = processor.suspendProcess(pids[j]);
				processor.delProcess(pids[j]);
				Processor newProcessor = scheduler.getHostToExecute();
				ProcPanel panel = monitor.getProcPanel(newProcess.getPID());
				panel.setHostLabel(newProcessor.getHostName());
				monitor.getMainPanel().writeLog("[SYSTEM] Migrating " + newProcess.getClassName() + " (PID " + newProcess.getPID() +") to " + newProcessor.getHostName());
				scheduler.migrateProcess(newProcess, newProcessor);
				ProcPanel procPanel = monitor.getProcPanel(newProcess.getPID());
				procPanel.setProcess(newProcess);
				monitor.getMenuBar().build();
			    }
			}
		    }
		}
		
		sleep(mseconds);
	    } catch (RemoteException e) {
		e.printStackTrace();
	    } catch (InterruptedException e) {
		e.printStackTrace();
	    }
	}
    }
}
