/****************************************************************************
   File:        MyMenuBar.java
   Description: MenuBar to be shown in the GUI.
   Project:     MAC 322 - Distributed Scheduler
   Authors:     Nelio Pereira (nelio@ime.usp.br)
   Institution: Department of Computer Science
                University of Sao Paulo
******************************************************************************/

package sod.server;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.rmi.RemoteException;
import javax.swing.*;

import sod.util.*;


public class MyMenuBar extends JMenuBar {

    private Monitor monitor;

    public MyMenuBar(Monitor monitor) {
        this.monitor = monitor;
	build();
    }

    public void build() {
	removeAll();
	
	JMenu menu;
	JMenuItem menuItem;
	
	menu = new JMenu("Tasks");
	menuItem = new JMenuItem("Main");
        menuItem.addActionListener(new ChangePanelAction(monitor.getMainPanel()));
	menu.add(menuItem);
	
        ListIterator listIterator = monitor.getProcesses().listIterator();
	while (listIterator.hasNext()) {
	    sod.util.Process proc = (sod.util.Process)listIterator.next();
	    int pid = proc.getPID();
	    String className = proc.getClassName();
	    String hostName = null;
	    try {
		hostName = proc.getProcessor().getHostName();
	    } catch (RemoteException e) {
		e.printStackTrace();
	    }
	    String text = className + "@" + hostName + ", PID=" + pid;
	    menuItem = new JMenuItem(text);
	    ProcPanel panel = monitor.getProcPanel(proc.getPID());
	    menuItem.addActionListener(new ChangePanelAction(panel));
	    menu.add(menuItem);
	}
	
	menu.addSeparator();
	menuItem = new JMenuItem("Exit");
	menuItem.addActionListener(new ExitAction(monitor));
	menu.add(menuItem);
	add(menu);
	validateTree();
    }

    class ChangePanelAction extends AbstractAction {
	
	private JPanel myPanel;
	
	public ChangePanelAction(JPanel panel) {
	    myPanel = panel;
	}

	public void actionPerformed(ActionEvent e) {
            monitor.changePanel(myPanel);
	}

    }

    class ExitAction extends AbstractAction {

	private Monitor monitor;

	public ExitAction(Monitor monitor) {
	    this.monitor = monitor;
	}

        public void actionPerformed(ActionEvent e) {
	    monitor.killProcessors();
            System.exit(0);
	}
    }

}
