/****************************************************************************
   File:        ProcPanel.java
   Description: ProcPanel is the panel that show processes information, and also, what it's printing to its OutputStream.
   Project:     MAC 322 - Distributed Scheduler
   Authors:     Nelio Pereira (nelio@ime.usp.br)
   Institution: Department of Computer Science
                University of Sao Paulo
******************************************************************************/

package sod.server;

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.*;
import javax.swing.*;
import java.rmi.RemoteException;

import sod.util.Process;
import sod.util.Processor;

public class ProcPanel extends JPanel implements ActionListener {

    public static String RUNNING_STATUS = "Running";
    public static String STOPPED_STATUS  = "Finished";

    public static String SEND_TEXT = "Go!";
    public static String CLOSE_TEXT = "Close";
    
    private Process process;
    private Monitor monitor;

    private JTextArea textArea;
    private JLabel statusLabel;
    private JLabel hostLabel;
    private JComboBox hostCombo;
    private JButton sendButton;

    public ProcPanel(Monitor monitor, Process process) { 
	super(true);
	this.monitor = monitor;
	setProcess(process);

	//	setBorder(BorderFactory.createEtchedBorder());
	setLayout(new BorderLayout());

	JPanel panel1 = new JPanel();
	//	panel1.setBorder(BorderFactory.createEtchedBorder());
	panel1.setLayout(new BorderLayout());

	JPanel panel3 = new JPanel();
	panel3.setBorder(BorderFactory.createEtchedBorder());
	panel3.setLayout(new BorderLayout());
	Color color = panel3.getForeground();
	panel3.setForeground(Color.blue);
	JLabel fileLabel = new JLabel("File: " + process.getClassFile().getAbsolutePath());
	panel3.add(fileLabel, BorderLayout.NORTH);
	String hostName = "";
	try {
	    hostName = process.getProcessor().getHostName();
	} catch (RemoteException e) {
	    e.printStackTrace();
	}
	setHostLabel(hostName);
	panel3.add(hostLabel, BorderLayout.CENTER);
	panel1.add(panel3, BorderLayout.NORTH);

	JPanel panel4 = new JPanel();
	panel4.setBorder(BorderFactory.createEtchedBorder());
	panel4.setLayout(new BorderLayout());
	panel4.add(new JLabel("Started running at " + new Date()), BorderLayout.NORTH);
	panel4.add(new JLabel("PID: " + process.getPID()), BorderLayout.CENTER);
	setStatusLabel(RUNNING_STATUS);
	panel4.add(statusLabel, BorderLayout.SOUTH);
	panel1.add(panel4, BorderLayout.CENTER);

	JPanel panel2 = new JPanel();
	panel2.setBorder(BorderFactory.createEtchedBorder());
	panel2.setLayout(new BorderLayout());
	JLabel sendTo = new JLabel("Send process to: ");
	panel2.add(sendTo, BorderLayout.WEST);

	//hostCombo = new JComboBox(scheduler.getHosts());
	hostCombo = new JComboBox();
	hostCombo.setEditable(false);
	panel2.add(hostCombo, BorderLayout.CENTER);
	sendButton = new JButton(SEND_TEXT);
	sendButton.addActionListener(this);
	panel2.add(sendButton, BorderLayout.EAST);
	panel1.add(panel2, BorderLayout.SOUTH);

	add(panel1, BorderLayout.NORTH);
	
	textArea = new JTextArea(); 
	textArea.setEditable(false);
	JScrollPane scroller = new JScrollPane();
	JViewport port = scroller.getViewport();
	port.add(textArea);
	port.setScrollMode(JViewport.BACKINGSTORE_SCROLL_MODE);
	add(scroller, BorderLayout.CENTER);
    }

    public void setHostLabel(String hostName) {
	String text = "Host: " + hostName;
	if (hostLabel == null) {
	    hostLabel = new JLabel(text);
	} else {
	    hostLabel.setText(text);
	}
    }

    public void setStatusLabel(String status) {
	String text = "Status: " + status; 
	if (statusLabel == null) {
	    statusLabel = new JLabel(text); 
	} else {
	    statusLabel.setText(text);
	}
    }

    public void buildHostList(String[] hosts) {
	hostCombo.removeAllItems();
	for (int i=0; i < hosts.length; i++) {
	    hostCombo.addItem(hosts[i]);
	}
    }

    public JTextArea getTextArea() {
	return textArea;
    }
    
    public Monitor getMonitor() {
	return monitor;
    }
    
    public void finish() {
	sendButton.setText(CLOSE_TEXT);
	hostCombo.setEnabled(false);
	setStatusLabel(STOPPED_STATUS + " (" + new Date() + ")");
	validate();
    }

    public void setProcess(Process process) {
	this.process = process;
    }

    public void actionPerformed(ActionEvent event) {
	if (event.getSource() == sendButton) {
	    int pid = process.getPID();
	    Processor processor = process.getProcessor();
	    String buttonText = sendButton.getText();
	    if (buttonText.compareTo(SEND_TEXT) == 0) {
		String hostName = (String)hostCombo.getSelectedItem();
		Process newProcess = null;
		try {
		    newProcess = processor.suspendProcess(pid);
		    processor.delProcess(pid);
		} catch (RemoteException e) {
		    e.printStackTrace();
		}
		monitor.migrateProcess(newProcess, hostName);
		setHostLabel(hostName);
		validate();
	    } else if (buttonText.compareTo(CLOSE_TEXT) == 0) {
		monitor.changePanel(monitor.getMainPanel());
		try {
		    processor.delProcess(pid);
		} catch (RemoteException e) {
		    e.printStackTrace();
		}
		monitor.delPanel(pid);
	    } else {
		System.err.println("Error in send button: unknow text ('" + buttonText + "')!");
	    }
	} else {
	    System.err.println("Error in event!");
	}
    }

}
