/****************************************************************************
   File:        Scheduler.java
   Description: All scheduler related items are hrer, such as policies, data structures and processes migration.
   Project:     MAC 322 - Distributed Scheduler
   Authors:     Nelio Pereira (nelio@ime.usp.br)
   Institution: Department of Computer Science
                University of Sao Paulo
******************************************************************************/

package sod.server;

import java.io.*;
import java.util.*;
import java.rmi.RemoteException;

import sod.util.Process;
import sod.util.Processor;


public class Scheduler {

    private Hashtable processors;

    public Scheduler() {
	processors = new Hashtable();
    }


    /**
     *
     * Return list of processes at all processors.
     */
    public List getProcesses() {
	Vector processes = new Vector();
	try {
	    for (Enumeration enum = processors.elements(); enum.hasMoreElements(); ) {
		Processor processor = (Processor)enum.nextElement();
		processes.addAll(processor.getProcesses());
	    }
	} catch (RemoteException e) {
	    e.printStackTrace();
	}
	return processes;
    }

    /**
     *
     * Return hosts that run processors.
     */
    public String[] getHosts() {
	String[] ret = new String[processors.size()];
	Vector v = new Vector(processors.keySet());
	v.toArray(ret);
	return ret;
    }

    public int getNumProcesses() {
	Vector processes = new Vector();
	int ret = 0;
	try {
	    for (Enumeration enum = processors.elements(); enum.hasMoreElements(); ) {
		Processor processor = (Processor)enum.nextElement();
		ret += processor.getNumProcesses();
	    }
	} catch (RemoteException e) {
	    e.printStackTrace();
	}
	return ret;
    }
    
    public synchronized Process createProcess(File file, Class classRef) {
	return new Process(file, classRef, getHostToExecute()); 	
    }

    public void execute(Process process) {
	try {
	    process.init();
	    Processor processor = process.getProcessor();
	    processor.execute(process);
	} catch (RemoteException e) {
	    e.printStackTrace();
	}
    }

    public void migrateProcess(Process process, Processor processor) {
	try {
	    process.setProcessor(processor);
	    processor.execute(process);
	} catch (RemoteException e) {
	    e.printStackTrace();
	}
    }

    public void migrateProcess(Process process, String hostName) {
	migrateProcess(process, (Processor)processors.get(hostName));
    }

    protected Processor getHostToExecute() {
	Enumeration procs = processors.elements();
	Processor ret = (Processor)procs.nextElement();
	try {
	    while (procs.hasMoreElements()) {
		Processor proc = (Processor)procs.nextElement();
		if (proc.getScore() < ret.getScore()) {
		    ret = proc;
		}
	    }
	} catch (RemoteException e) {
	    e.printStackTrace();
	}
	return ret;
    }

    public synchronized void addProcessor(Processor processor) {
	try {
	    int size = processors.size();
	    processors.put(processor.getHostName(), processor);
	} catch (RemoteException e) {
	    e.printStackTrace();
	}
    }

    public synchronized void delProcessor(String hostName) {
	processors.remove(hostName);
    }

    public synchronized void delProcessor(Processor processor) {
	try {
	    processors.remove(processor.getHostName());
	} catch (RemoteException e) {
	    e.printStackTrace();
	}
    }

    public Processor getProcessor(String hostName) {
	return (Processor)processors.get(hostName);
    }

    public Vector getProcessors() {
	return new Vector(processors.values());
    }
}

