/****************************************************************************
   File:        AbstractApplication.java
   Description: An abstract class representing an application that may be migrated.
   Project:     MAC 322 - Distributed Scheduler
   Authors:     Nelio Pereira (nelio@ime.usp.br)
   Institution: Department of Computer Science
                University of Sao Paulo
******************************************************************************/

package sod.util;

import java.io.*;

/**
 *
 * This is the superclass of any class that wants to be executed.
 */
public abstract class AbstractApplication implements Serializable {

    protected OutputStream out;
    
    /**
     * 
     * This common methods initializes the reference of the class OutputStream.
     */
    public void init(OutputStream out) {
	this.out = out;
    }

    /**
     *
     * This method will be called to run this application. Note that it will be called even after the application was suspended.
     */
    public abstract void run();


    /**
     *
     * This method will be called to suspend the application, so the process can be migrated.
     */
    public abstract void suspend();


    /**
     *
     * This method will be called to resume execution of the application after a it was suspended.
     */
    public abstract void resume();
   
}
