/****************************************************************************
   File:        Process.java
   Description: Representation of a distributed process.
   Project:     MAC 322 - Distributed Scheduler
   Authors:     Nelio Pereira (nelio@ime.usp.br)
   Institution: Department of Computer Science
                University of Sao Paulo
******************************************************************************/

package sod.util;

import java.io.*;
import java.lang.reflect.*;
import javax.swing.*;   
import java.rmi.Naming;
import java.rmi.RemoteException;

/**
 *
 * Representation of a distributed process.
 */
public class Process extends Thread implements Serializable {
    
    private static int next_pid = 1;
    
    private File classFile;
    private int pid;
    private AbstractApplication application;
    private Processor processor;
    private SchedulerUI schedulerUI;
    private Class classRef;
    private boolean started;
    
    public Process(File classFile, Class classRef, Processor processor) {
	this.pid = next_pid++;
	this.classFile = classFile;
	this.processor = processor;
	this.classRef = classRef;
	started = false;
    }

    
    /**
     *
     * Initialize the executable object.
     */
    public void init() { 
        try {
	    application = (AbstractApplication)classRef.newInstance();
	    Class[] argsC = new Class[1];
	    argsC[0] = Class.forName("java.io.OutputStream");
	    Method instanceMethod = classRef.getMethod("init", argsC);
	    Object[] argsO = new Object[1];
	    try {
		schedulerUI = (SchedulerUI)Naming.lookup("//" + processor.getServerName() + "/SchedulerUI" + pid); 
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	    argsO[0] = new LoggerOutputStream(schedulerUI);
	    instanceMethod.invoke(application, argsO);
        } catch (Exception e) {
	    e.printStackTrace();
        }
    }

    public int getPID() {
	return pid;
    }

    public File getClassFile() {
	return classFile;
    }

    public String getClassName() {
	String fileName = classFile.getName();
	return fileName;
    }

    public Processor getProcessor() {
	return processor;
    }

    public void setProcessor(Processor processor) {
	this.processor = processor;
    }

    public void run() {
	started = true;
        application.run();
	try {
	    schedulerUI.finish();
	    schedulerUI.writeAll("Finished execution of " + getClassName() + " (PID=" + pid + ")");
	} catch (RemoteException e) {
	    e.printStackTrace();
	}
    }

    public void stopExecution() {
	application.suspend();
	super.stop();
	try {
	    schedulerUI.writeAll("Suspended execution of " + getClassName() + " (PID=" + pid + ")");
	} catch (RemoteException e) {
	    e.printStackTrace();
	}
    }

    public void resumeExecution() {
	application.resume();
	try {
	    schedulerUI.writeAll("Resumed execution of " + getClassName() + " (PID=" + pid + ")");
	} catch (RemoteException e) {
	    e.printStackTrace();
	}
    }

    public boolean isStarted() {
	return started;
    }
}
